import java.awt.*;
import java.applet.*;
import javax.swing.*;
import javax.swing.table.*;
import java.util.*;
import javax.swing.event.*;

/**
 * BinaryConverter.java 1.0 02/11/01
 * <br>
 * Author       : James McMahon
 * <br>
 * BinaryConverter Applet Class
 * <br>
 * The Panel that contains the answer table and 
 * deals with working out whether it's correct or not.
 * <br>
 * Unlike the other applets which have a separated out Score Panel,
 * this applet deals with the score panel inside here
 * <br>
 * This could probably be modified to separate it out, but is relatively pointless
 * <br>
 * Created      : 1/11/2001
 * <br>
 * Last Updated : 1/11/2001
 */

public class AnswerTable extends JPanel {
	
	/**
	 * The parent of this class needed because we tell the user of errors via JOptionPanes 
	 * which are model to the Applet frame
	 */
	private BinaryConverter parent; 
	
	//Score Tracker variables and label
	
	/**
	 * The number of questions asked
	 */
	private int 			numAsked 	= 0;  
	
	/**
	 * The number of questions answered correctly
	 */
	private int 			numRight	= 0; 
	
	/**
	 * The label that is used to display the user's score
	 */
	private JLabel			score		= new JLabel(""+numRight+" correct out of "+numAsked+"  ");

	/** 
	 * The JTable that is used to hold the users answer
	 */
	private JTable			ansTable;
	
	//Panels to hold both the table and the score card	
	
	/**
	 *
	 */
	private JScrollPane		tableHolder; //initilised with an instance of the table
	
	/**
	 *
	 */
	private JPanel			scoreHolder	 = new JPanel();
	
	/**
	 * Constructs the answer table
	 * 
	 * @param parentOwner The main applet class which is needed to make the JOptionPanes model
	 */
	public AnswerTable(BinaryConverter parentOwner) {
	
		parent = parentOwner;
		
		this.setLayout(new BorderLayout());
		this.setPreferredSize(new Dimension(100,75));
	
		setUpTable();
			
		//Add the table and score tracker to the JPanel
		tableHolder = new JScrollPane(ansTable);
		tableHolder.setBorder(BorderFactory.createTitledBorder("Answer Table"));
		
		scoreHolder.add(score);
		scoreHolder.setBorder(BorderFactory.createTitledBorder("Score Tracker"));
	
		this.add(tableHolder, BorderLayout.CENTER);
		this.add(scoreHolder, BorderLayout.EAST);
	}	
	
	/**
	 * Sets up the table that is needed to hold the answers from the user
	 */
	public void setUpTable(){
		AnswerTableModel tableModel = new AnswerTableModel(parent);
			
		ansTable = new JTable(tableModel);
		ansTable.setRowSelectionAllowed(true);
		ansTable.setColumnSelectionAllowed(true);
		
		ansTable.getTableHeader().setReorderingAllowed(false);//does the same job as the columnmodel listener used to do
		ansTable.getTableHeader().setResizingAllowed(false);

		//following allows cell editing to begin with the text highlighted
		ListSelectionListener cellListener = new CellListener(ansTable);	
		ansTable.getSelectionModel().addListSelectionListener(cellListener);
		ansTable.getColumnModel().getSelectionModel().addListSelectionListener(cellListener);
		
		//Make a single click to be the point that cell editing begins
		DefaultCellEditor dce = new DefaultCellEditor(new JTextField());
		
		for(int i=0; i<ansTable.getColumnCount(); i++) {
			ansTable.getColumnModel().getColumn(i).setCellEditor(dce);
			dce.setClickCountToStart(0);
		}		
		
		setUpToolTips();
	}
	
	/**
	 * Sets up the tool tips for the table
	 */
	private void setUpToolTips() {
		
		int power = 7;
		for(int i=0; i<=7; i++) {//loop through all the columns giving them tooltips	
			
			TableColumn column = ansTable.getColumnModel().getColumn(i);
		
			//Set up tool tips for the cells.
       	 	DefaultTableCellRenderer renderer = new CellRenderer();
        	renderer.setToolTipText("2^"+power);
        	column.setCellRenderer(renderer);

        	//Set up tool tip for the cell header.
        	TableCellRenderer headerRenderer = ansTable.getDefaultRenderer(ansTable.getModel().getColumnClass(i));
        	if (headerRenderer instanceof DefaultTableCellRenderer) {
         	   ((DefaultTableCellRenderer)headerRenderer).setToolTipText("2^"+power);
        	} 
        	power -= 1; //reduce power for next time around loop
		}
	}
		
	/** 
	 * Add one to the numer of questions asked
	 * <br>
	 * Gets called when generate question is called	
	 */
	public void addToNumAsked() {
		numAsked += 1;
		//generate the string
		score.setText(""+numRight+" correct out of "+numAsked+"  "); 
	}	
		 
	/** 
	 * Add one to the score count
	 * <br>
	 * Gets called when checkAnswer is called	
	 */
	public void addToScore() {
		numRight += 1;
		//generate the string
		score.setText(""+numRight+" correct out of "+numAsked+"  "); 
	}
	
	/** 
	 * Reset score to 0 out of 0
	 */
	public void resetScore() {
		numAsked = 1; //Because there is still one being displayed
		numRight = 0;
		score.setText(""+numRight+" correct out of "+numAsked+"  ");
	}
	
	/**
	 * Set the answer in the table to the correct value
	 */
	public void showCorrectAnswer(){
		String answer = parent.getQuestionPanel().getAnswer();
		//answer must be a string of length 8 containing all of the valid binary value for the table

		for( int i = 0; i<answer.length();i++) {
			ansTable.setValueAt(answer.substring(i,i+1), 0, i);	
		}
		parent.getWorkSpacePanel().showFullWorking();//completes the workspace screen as well	
	}
	
	/** 
	 * Removes focus from any currently editing cells.
	 * <br>
	 * Used primarily in check answer to ensure that the user doesn't input an answer
	 * and forget to manually remove focus, thereby causing an incorrect value 
	 * in the cell to be evaluated
	 */
	private void removeEditingFocus() {
		int row = ansTable.getSelectedRow();
		int col = ansTable.getSelectedColumn();
		if (ansTable.isEditing()) {
			ansTable.getCellEditor(row,col).stopCellEditing(); 
		}
	}
	
	/** 
	 * Check the answer which is currently held in the Table
	 * <br>
	 * Updates score by 1 if the result returned is true
	 *
	 * @return true If the user's answer is correct and matches the computers one 
	 * @return false If they don't match
	 */
	public boolean checkAnswer() {
		
		String       answer         = parent.getQuestionPanel().getAnswer();
		StringBuffer studentAnswer  = new StringBuffer("");
		StringBuffer emptyLocations = new StringBuffer("");//holds the location of any '?' in the table
		
		//first thing to do is force the focus away from all of the JTable cells
		removeEditingFocus();

		
		//Check through all of row zero to build up the answer
		for (int i = 0 ; i < ansTable.getColumnCount() ; i++){
			studentAnswer.append(ansTable.getValueAt(0,i));
		}
		
		//Only check the answer if it has all been filled in		
		//i.e. No ? values
		boolean firstValue = true; //ensures sensible message in dialog box
		
		for (int i = 0 ; i < studentAnswer.length(); i++){
			if (studentAnswer.charAt(i)=='?') {
				if (firstValue) {//first time give this to the message string
					emptyLocations.append((i+1));
					firstValue = false;
				} else {
					emptyLocations.append(", "+(i+1));
				}	
			}
		}	
		
		if (emptyLocations.length() == 0) {//User has a complete answer so check it
			if (studentAnswer.toString().equals(answer)) {
				addToScore();//Add one to the score
				return true;
			}
			else { 			
				return false;//Answer was wrong
			}
		}
		else if (emptyLocations.length() == 1) { //User has left one part of the table incomplete, so tell them
			JOptionPane.showMessageDialog(parent,
			   				  "You need to complete the cell at location "+emptyLocations,
				              "Missing Cell Information",
				              JOptionPane.WARNING_MESSAGE);	
			return false;//answer wasn't right so return false
		}
		else { //User has left more than one part of the table incomplete, so tell them
			JOptionPane.showMessageDialog(parent,
			   				  "You need to complete the cells at locations "+emptyLocations,
				              "Missing Cell Information",
				              JOptionPane.WARNING_MESSAGE);	
			return false;//answer wasn't right so return false
		}
	}
	
	/**
	 *	Clear the table fields to question marks
	 */
	public void clearFields() {	
		//Loops through each cell in row 1 and 
		for(int i=0; i<ansTable.getColumnCount();i++) {
			ansTable.setValueAt("?",0,i);
		}	
	} 
}

//ensures clicking on a table cell results in the text being automatically highlighted
class CellListener implements ListSelectionListener {
		
	/**
	 * The ansTable that the cell renderer is to be set for
	 */
	private JTable ansTable; 
	
	/**
	 * Creates a cell listener so that the selected cell can have it's text highlighted
	 * 
	 * @param table The table that is to be listened to
	 */
	public CellListener(JTable table) {
		ansTable = table;
	}
		
	/**
	 * Ensures that when the table cells receive focus then the text inside is highlighted
	 * @param ListSelectionEvent The event generated by the table when a cell is selected
	 */
	public void valueChanged(ListSelectionEvent e) {
    	ListSelectionModel lsm = (ListSelectionModel)e.getSource();
     	int col = ansTable.getSelectedColumn();
     	int row = ansTable.getSelectedRow();

    	if (row >= 0) { 
   	 		Component comp = ansTable.getCellEditor(row,col).getTableCellEditorComponent(ansTable, ansTable.getValueAt(row,col), true,row,col); 
    		((javax.swing.JTextField)comp).select(0, 1);
    	}
    }
}

//Ensures that when a cell recieves Focus, then it automatically recieves editing status as well
//(so that focusing exhibits the same behaviour of selecting internal cell text, as editing does)
class CellRenderer extends DefaultTableCellRenderer {
	
	/**
	 * ensures that the answer table cells have the text highlighted when it is selected
	 * and not only when it recieves focus
	 * 
	 * @return Component The cell component with the internal text highlighted
	 */
	public Component getTableCellRendererComponent(JTable ansTable,
												   Object value,
												   boolean isSelected,
												   boolean hasFocus,
												   int row, int col) {

		//this does the same thing as the list selection listener
		//Essentially it turns any cell who gets focus automatically into an editing cell 
		if (hasFocus) {
	
		ansTable.editCellAt(row,col);
     
     	int col1 = ansTable.getSelectedColumn();
     	int row1 = ansTable.getSelectedRow();

    		if (row1 >= 0) { 
   	 			Component comp = ansTable.getCellEditor(row1,col1).getTableCellEditorComponent(ansTable, ansTable.getValueAt(row1,col1), true,row1,col1); 
    			((javax.swing.JTextField)comp).select(0, 1);	
    		}		
		}
		
		super.getTableCellRendererComponent(ansTable,value,isSelected,hasFocus,row,col);
		return this;
	}
}