import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import javax.swing.JScrollPane;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;
import javax.swing.JOptionPane;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 *
 * Modified code from the TableModel example on java.sun.com
 * 
 */

public class AnswerTableModel extends AbstractTableModel {
  
  	/** 
  	 * The parent applet class
  	 * <br>
  	 * Needed because we have a JOptionPane that is model to the parent applet
  	 */
  	private BinaryConverter parent;
  	
  	/**
  	 *
  	 */
	final String[]   columnNames = {"128","64","32","16","8","4","2","1"};
	
	/**
	 *
	 */
	final Object[][] data 	   = {{"?","?","?","?","?","?","?","?"}}; 

	/**
	 * Constructs an answer table model with the parent class
	 *
	 * @param parentOwner The parent applet class for use with the JOptionPanes
	 */
   	public AnswerTableModel(BinaryConverter parentOwner) {
  		parent = parentOwner;
  	}

	/**
	 * Returns the number of columns in the table
	 * @return int The number of columns
	 */
    public int getColumnCount() {
         return columnNames.length;
    }
        
 	/**
	 * Returns the number of rows in the table
	 * @return int The number of rows
	 */      
    public int getRowCount() {
         return data.length;
    }
	
 	/**
	 * Returns the name of the column col in the table
	 * 
	 * @param col The column that the name of is to be returned
	 * @return String The name of the column
	 */    
	public String getColumnName(int col) {
         return columnNames[col];
    }

 	/**
	 * Returns the value that is in the cell at location row,col
	 * 
	 * @param row The row of the cell wanted
	 * @param col The column of the cell wanted
	 *
	 * @return Object The value of the cell at the specified location
	 */    
    public Object getValueAt(int row, int col) {
         return data[row][col];
    }

    /**
     * JTable uses this method to determine the default renderer/
     * editor for each cell.  If we didn't implement this method,
     * then the last column would contain text ("true"/"false"),
     * rather than a check box.
     */
    public Class getColumnClass(int c) {
         return getValueAt(0, c).getClass();
    }

    /**
     * Don't need to implement this method unless your table's
     * editable.
     */
    public boolean isCellEditable(int row, int col) {
     //Note that the data/cell address is constant,
     //no matter where the cell appears onscreen.
         return true; //All cells are editable
    }

    /**
     * Don't need to implement this method unless your table's
     * data can change.
     * <br>
     * Only allows 0,1 or ? values
     */
    public void setValueAt(Object value, int row, int col) {
	//MODIFIED		
	//Only allows 1,0 or ? to be set in a cell
				
         if (value.toString().equals("0") 
             | value.toString().equals("1")
             | value.toString().equals("?")) {
            
     	 	data[row][col] = value;
            fireTableCellUpdated(row, col);
         } 
         else {
         	JOptionPane.showMessageDialog(parent,
            	        "You must enter either a 0 or a 1",
            	        "Invalid Input",
						JOptionPane.INFORMATION_MESSAGE); 
            }
        }
}