import java.awt.*;
import java.applet.*;
import javax.swing.*;

/**
 * BinaryConverter.java 1.0 02/11/01
 * <br>
 * Author       : James McMahon
 * <br>
 * BinaryConverter Applet Class
 * <br>
 * Created      : 1/11/2001
 * <br>
 * Last Updated : 1/11/2001
 * <br> <br>
 * N.B. Due to the nature of the creation of the panels, 
 * access to each panel needs to be available to all others because it
 * is currently impossible to instantiate a specific panel without first 
 * having the others ones available to pass in! Hence call's to other panels 
 * are only ever accessed after every other one is created.
 * <br>
 */

public class BinaryConverter extends JApplet {
	
	/**
	 * The Menu Bar Panel
	 */
	private MenuPanel 	  	menuPanel 	  = new MenuPanel(this);
	
	/**
	 * The question area Panel for displaying and generating questions
	 */
	private QuestionArea    questionPanel = new QuestionArea(this); 

 	/**
 	 * The WorkSpace area Panel where the user can try out the answer
 	 */
	private TempWorkSpace	tempWork	  = new TempWorkSpace(this);
	
	/**
	 * Answer Table Panel (also has the scorepanel inside)
	 */
	private AnswerTable		ansTable	  = new AnswerTable(this); 
	
	/**
	 *  Panel to contain all of the the other panels
	 */
	private JPanel 			containPanel = new JPanel();

	/**
	 * The main class of an applet
	 * <br>
	 * Sets up a question to start with
	 */
	public void init() {
		
		super.init();

		questionPanel.generateQuestion();//Generate a question to start with	
		addAllPanels(); // Put all the panels on to the screen and let control pass to the MenuListener
	
	}
	
	/**
	 * Adds all of the panels on to the applets contentPane
	 */
	public void addAllPanels() {
		
		getContentPane().setLayout(new BorderLayout()); //Set the applet to BorderLayout
		containPanel.setLayout(new BorderLayout());     //Set the sub panel to BorderLayout

		getContentPane().add(questionPanel, BorderLayout.NORTH);//Add the question panel to the top of the applet
		getContentPane().add(menuPanel, BorderLayout.SOUTH);//Add the menu panel to the bottom of the applet
					
		containPanel.add(tempWork, BorderLayout.CENTER);//Add the work space panel to the middle of the sub panel
		containPanel.add(ansTable, BorderLayout.SOUTH);//Add the answer space panel to the bottom of the sub panel		

		getContentPane().add(containPanel, BorderLayout.CENTER); //Add the sub panel to the applet 
	}
	
	/**
	 * Doesn't do much as there are no explicit threads created in this applet
	 */
	public void start()	{
		super.start();
	}
	
	/**
	 * Doesn't do much as there are no explicit threads created in this applet
	 */
	public void stop() {
		super.stop();
	}
	
	/** 
	 * Clears all of the TextFields and Tables so that the new question has a clean workspace
	 */
	public void clearAllTables() {
		getAnswerTablePanel().clearFields();
		getWorkSpacePanel().clearFields();
	}
	
	/** 
	 * Clears all the panels and resets the score to zero
	 */
	public void clearAll() {
		clearAllTables();
		getAnswerTablePanel().resetScore();	
	}
	
	/** 
	 * Accessors for all of the panels
	 * <br>
	 * @return AnswerTable The answer table
	 */
	public AnswerTable getAnswerTablePanel() {
		return ansTable;	
	}
	
	/** 
	 * Accessors for all of the panels
	 * <br>
	 * @return QuestionArea The question area panel
	 */	
	public QuestionArea getQuestionPanel() {
		return questionPanel;	
	}
	
	/** 
	 * Accessors for all of the panels
	 * <br>
	 * @return TempWorkSpace The work space panel provided for the user to try answers and see hints on
	 */
	public TempWorkSpace getWorkSpacePanel() {
		return tempWork;	
	}
	
	/** 
	 * Accessors for all of the panels
	 * <br>
	 * @return MenuPanel The Menu Panel from the bottom of the screen
	 */
	public MenuPanel getMenuPanel() {
		return menuPanel;	
	}	
}