import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * BinaryConverter.java 1.0 02/11/01
 * <br>
 * Author       : James McMahon
 * <br>
 * BinaryConverter Applet Class
 * <br>
 * The MenuListener to deal with all of the necessary actions 
 * <br>
 * N.B. Currently tightly coupled to the labels of the JButtons in the MenuPanel
 * - change one then remember to change the other
 * <br>
 * Created      : 1/11/2001
 * <br>
 * Last Updated : 1/11/2001
 */

public class MenuListener implements ActionListener {
	
	/**
	 * Necessary because we again have a JOptionPane model to the main applet
	 */
	private BinaryConverter parent;
	
	/**
	 * Helps stop cheating with the score tracker by 
	 * storing what the user has already tried to do
	 */
	private boolean cheater       = false;

	/**
	 * Helps give meaningful messages to the user if they submit a wrong answer
	 */
	private boolean helpedAlready = false;
	
	/** 
	 * Helps stop user's being able to submit the same answer more than once
	 */
	private boolean submittedAnswerAlready = false; 
	
	/**
	 * Constructs the menu listener with the parent applet class used with the JOptionPanes
	 *
	 * @param parentOwner The main applet class
	 */
	public MenuListener(BinaryConverter parentOwner) {
		parent = parentOwner;
	}
	
	/**
	 * Deals with all of the actions that are associated with all of the buutons on the menu panel
	 *
	 * @param ActionEvent The event that needs to be dealt with
	 */
	public void actionPerformed(ActionEvent ae) {
	
		if (ae.getActionCommand().equals("New")) {
		//Generates new question and clears the junk from the old one away
			parent.getQuestionPanel().generateQuestion();
			
			parent.clearAllTables();
			cheater = false;       //new question so not cheating
			helpedAlready = false; //new question so not been helped yet
			submittedAnswerAlready = false;//new question so not submitted answer yet
		}
		else if (ae.getActionCommand().equals("Help")) { 
			HelpPanel helper = new HelpPanel(parent);		
		}
		else if (ae.getActionCommand().equals("Hint")) {
			if (cheater) { //then they have already been given the answer so no need for hint
					
			JOptionPane.showMessageDialog(parent,
							  "The answer is already in the table..., Please try a new question", 
							  "Answer Already Given",
							  JOptionPane.INFORMATION_MESSAGE);
			} 
			else { //Give them the hint
			parent.getWorkSpacePanel().giveHint();
			helpedAlready = true; //They have now been given help for this question
			}
		}
		else if (ae.getActionCommand().equals("Reveal")) { 
			parent.getAnswerTablePanel().showCorrectAnswer();
			
			cheater = true; //they have been given answer so they could be a cheater
			
			// Give the user a dialog explaining the fact the answer is now in 
			// the table and they can try a new question if they want
			JOptionPane.showMessageDialog(parent,
										  "The answer is in the table..., Please try a new question", 
										  "Answer Completed",
										  JOptionPane.INFORMATION_MESSAGE);
		}
		else if (ae.getActionCommand().equals("Submit")) { 
		
			if (cheater | submittedAnswerAlready) {//the user is trying to cheat so don't let him!
		  	
		  		if (cheater) {//User tried to cheat by using computer's answer
					JOptionPane.showMessageDialog(parent,
											  "Don't you trust the answer I gave you?",
									          "Thinking of cheating?",
									          JOptionPane.ERROR_MESSAGE);  	
		 		}
		 		else {//
		 			JOptionPane.showMessageDialog(parent,
											  "You've already submitted this answer! Use the NEW button for a new question to try",
									          "Thinking of cheating?",
									          JOptionPane.ERROR_MESSAGE);  	
		 		}
		 	}
		 	else { //not a cheater so they can have the answer checked
				if (parent.getAnswerTablePanel().checkAnswer()) { //answer was correct, will automatically add 1 to score
					//Correct 
					submittedAnswerAlready = true;//make sure they don't submit it again
					JOptionPane.showMessageDialog(parent,
											  "Why don't you try another question",
									          "Well Done: Correct Answer",
									          JOptionPane.INFORMATION_MESSAGE);
				}
				else {
					if (helpedAlready) {//tell them to use reveal
						JOptionPane.showMessageDialog(parent,
						    				  "If you're really stuck then use reveal",
								              "Sorry: Incorrect Answer",
								              JOptionPane.WARNING_MESSAGE);
					}
					else {//not tried hint yet so tell them to use it
						JOptionPane.showMessageDialog(parent,
						    				  "Why not try hint to get some ideas?",
								              "Sorry: Incorrect Answer",
								              JOptionPane.WARNING_MESSAGE);
					}
				}
		 	}
		}
		else if (ae.getActionCommand().equals("Reset")) { 
			parent.clearAllTables(); //Clears all of the tables
			//parent.clearAll(); //Clears all tables plus resets score!
		}
		else if (ae.getActionCommand().equals("Refresh")) { 
			parent.repaint();
		}
	}
}