import java.awt.*;
import java.applet.*;
import javax.swing.*;

/**
 * BinaryConverter.java 1.0 02/11/01
 * <br>
 * Author       : James McMahon
 * <br>
 * BinaryConverter Applet Class
 * <br>
 * The Menu Pannel for the applet menu
 * <br>
 * Created      : 1/11/2001
 * <br>
 * Last Updated : 1/11/2001
 */

public class MenuPanel extends JPanel {
	
		/**
		 * The parent of this class.<br>
		 * Used exclusively to be passed to the MenuListener<br>
		 * It is not used to access any other workpanels!
		 */
		private BinaryConverter parent; 	
	
		/**
		 * Get a new question button
		 */
		private JButton newQuestion = new JButton("New");
		
		/**
		 * Display a help page button
		 */
		private JButton help        = new JButton("Help");
		
		/**
		 *  Show a hint, by filling part of the temp workspace button
		 */
		private JButton hint		= new JButton("Hint");
		
		/**
		 * Reveal the answer and pass the question by button
		 */ 
		private JButton reveal      = new JButton("Reveal");
		
		/**
		 * Check the user's answer is correct button
		 */
		private JButton check	    = new JButton("Submit"); 
		
		/**
		 * Reset the Score button
		 */
		private JButton reset		= new JButton("Reset");
		
		/**
		 * Button to allow the screen to be refreshed if necessary
		 */
		private JButton refresh     = new JButton("Refresh");
		
		/**
		 * The listener for the buttons
		 */
		private MenuListener mListener;	
	
		/**
		 *  Sets up the menu panel with the buttons. <br>
		 *  The parent applet class is used to pass on to the menu listener
		 *  
		 *  @param parentOwner The main applet class for passing on to the menu listener
		 */
		public MenuPanel(BinaryConverter parentOwner) {
		
			parent = parentOwner;
		
			this.setLayout(new FlowLayout());
			this.setBorder(BorderFactory.createEtchedBorder());
			
			addListeners(); //Add the MenuListener to all of the buttons
		
			//Add the tooltips to the buttons
			newQuestion.setToolTipText("Get's a new question");
			help.setToolTipText("Displays a help page");
			hint.setToolTipText("Get a hint on the answer");
			reveal.setToolTipText("Displays the correct answer");
			check.setToolTipText("Check if your answer is correct");
			reset.setToolTipText("Reset panels to zero ");
			refresh.setToolTipText("Refresh the display");
			
			newQuestion.setMnemonic('n');
			help.setMnemonic('h');
			hint.setMnemonic('i');
			reveal.setMnemonic('r');
			check.setMnemonic('s');
			reset.setMnemonic('t');
			refresh.setMnemonic('f');
			
			// Add the buttons to the panel
			this.add(newQuestion);
			this.add(help);
			this.add(hint);
			this.add(reveal);
			this.add(check);
			this.add(reset);
			this.add(refresh);
		}	
	
		/**
		 * Adds the menu listeners on to all of the menu buttons
		 */
		public void addListeners() {
		
			mListener = new MenuListener(parent);
		
			newQuestion.addActionListener(mListener); 
			help.addActionListener(mListener);     
			hint.addActionListener(mListener);
			reveal.addActionListener(mListener);
			check.addActionListener(mListener);
			reset.addActionListener(mListener);
			refresh.addActionListener(mListener);
		}
}