import java.awt.*;
import java.applet.*;
import java.util.*;
import javax.swing.*;

/**
 * BinaryConverter.java 1.0 02/11/01
 * <br>
 * Author       : James McMahon
 * <br>
 * BinaryConverter Applet Class
 * <br>
 * Question Area that generates and displays the questions
 * <br>
 * Created      : 1/11/2001
 * <br>
 * Last Updated : 1/11/2001
 */

public class QuestionArea extends JPanel {
	
	/** 
	 * The parent of this class
	 * <br>
	 * Used to access the AnswerTable class for updating the number of questions asked only!
	 */
	private BinaryConverter parent; 
	
	/**
	 * The first part of the string for use with the question
	 */
	private String partOne = "Can you convert the decimal number ";
	
	/**
	 * The second part of the string for use with the question
	 */
	private String partTwo = " to binary representation?";
	
	/**
	 * The actual question being asked combined into one textfield
	 */
	private JTextField question = new JTextField(partOne+"X"+partTwo, 40);
	
	/**
	 * The number that the question asks to be converted
	 */
	private int number; 
	
	/** 
	 * The answer string that this question evaluates to
	 * <br>
	 * N.B. provided here because calling calcAnswer muliple
	 * times is very wasteful - so now it gets called once, accessed many
	 */	
	private String answer; 
	
	/**
	 * Sets up the question panel area, and sets the parentOwner of the class
	 *
	 * @param parentOwner The main applet class used to access the other panels (primarily the score tracker panel)
	 */
	public QuestionArea(BinaryConverter parentOwner) {
	
		parent = parentOwner;
	
		this.setLayout(new FlowLayout());
		this.setBorder(BorderFactory.createTitledBorder("Please answer the following"));
		
		question.setEditable(false);
		this.add(question);
	}	
	
	/** 
	 * Generates a question -- displays the question and stores the result in number
	 */
	public void generateQuestion() {
		
		Random randNum = new Random();
		number = randNum.nextInt(255); //create random number hopefully between 0..255 and store it
		
		parent.getAnswerTablePanel().addToNumAsked();//asked new question so update score
		
		question.setText(partOne+number+partTwo);//set the question text
		answer = calculateAnswerToQuestion();//set the question answer as well
	}
	
	/** 
	 * Calculates the result of converting the current variable held in number into binary
	 * returns a string containing the results of the conversion like 10101011 for example
	 * <br>
	 * Originally SCOTT'S CODE but modified for use in this program
	 */
	private String calculateAnswerToQuestion() {
		
		int power = 7;
		int base  = 2;
		int numberCopy = number;
		
		StringBuffer answerBuffer = new StringBuffer();
		
		while (power >= 0){
			int sum = calcPower(base,power);
			if (numberCopy >= sum){
				answerBuffer.append("1");
				numberCopy = numberCopy - sum;
			}
			else
				answerBuffer.append("0");
					
			power--;
		}
		return answerBuffer.toString();
	}
	
	/**
	 * Returns b to the power n (providing b is non negative integer).
	 * This uses a smart power algorithmn to quickly calculate the maximum value
 	 * for any number to the power of another number. 
  	 * <br>
	 * Originally SCOTT'S CODE
	 */
	private int calcPower(int b, int n)	{
		int p = 1;
		int q = b;
		int m = n;
		
		while (m>0)	{
			if (m%2 !=0)
				p*=q;
			m/=2;
			q*=q;
		}
		return 	p;
	}

	/** 
	 * returns the answer for this question
	 * 
	 * @return String The answer term for this question (i.e 00001100 or 00000000 etc)
	 */
	public String getAnswer() {
		return answer;
	} 
	
	/** 
	 * Returns the number that is to be converted for this particular question.
	 * 
	 * @return int An int in the range 0 to 255
	 */
	public int getNumberForConversion() {
		return number;
	}
}